@@ -20,6 +20,8 @@ class postprocess:
         subbasingroup_length_channel=[-1],
         subbasingroup_name_lake=["AllLakesubbasins"],
         subbasingroup_area_lake=[-1],
+        lake_out_flow_method = 'broad_crest',
+        model_time_step = 1,
     ):
         """This function is to generate Raven input files. A subfolder 'RavenInput'
         in path_output_folder is created to sotre resultant outputs.
@@ -113,7 +115,7 @@ class postprocess:
             SubBasinGroup_Area_Lake=subbasingroup_area_lake ,
             OutputFolder=path_output_folder,
             Forcing_Input_File=forcing_input_file,
-            aspect_from_gis = aspect_from_gis
+            aspect_from_gis = aspect_from_gis,
         )
 
 
@@ -610,7 +612,9 @@ class postprocess:
         Other_Ply_ID_1="O_ID_1"
         Veg_ID="Veg_ID"
         Other_Ply_ID_2="O_ID_2"
-
+        
+        os.environ["GISDBASE"] = path_output_folder
+        
         generate_hrus_method(
             Path_Subbasin_Ply=path_subbasin_polygon,
             Landuse_info=path_landuse_info,
@@ -828,6 +832,7 @@ class delineate:
         fac_thresold,
         mode = 'using_dem',
         path_flow_dirction="#",
+        path_flow_accumulation='#',
         max_memroy=1024 * 4,
         gis_platform="qgis",
         subreg_fdr_path="#",
@@ -892,6 +897,8 @@ class delineate:
             mode = 'usingdem'
         elif mode == 'using_sub_reg':
             mode = 'usingsubreg' 
+        elif mode == 'using_fac':
+            mode = 'usingfac'
         else:
             mode = 'usingfdr'
 
@@ -900,6 +907,7 @@ class delineate:
             input_geo_names=self.geofilenames,
             acc_thresold=fac_thresold,
             fdr_path=path_flow_dirction,
+#            fac_path = path_flow_accumulation,
             subreg_fdr_path=subreg_fdr_path,
             subreg_acc_path=subreg_acc_path,
             subreg_str_r_path=subreg_str_r_path,
