@@ -6,6 +6,286 @@ import qgis
 from qgis.analysis import QgsNativeAlgorithms
 from qgis.core import *
 from qgis.PyQt.QtCore import *
+from joblib import Parallel, delayed
+from basinmaker.utilities.utilities import *
+import tempfile
+from json import load, JSONEncoder
+import json
+
+####
+####
+def create_geo_jason_file(processing,Input_Polygon_path):
+
+
+    product_dir = os.path.dirname(Input_Polygon_path)
+    Names_in = os.path.basename(Input_Polygon_path).split('_')
+    n_charc = len(Names_in)
+    version  = Names_in[n_charc - 1][0:4]
+    TOLERANCEs = [0.0001,0.0005,0.001,0.005,0.01,0.05,0.1]
+
+
+    head_name_cat = "finalcat_info"
+    head_name_riv = "finalcat_info_riv"
+    head_name_slake = "sl_connected_lake"
+    head_name_nlake = "sl_non_connected_lake"
+
+    Input_file_name = []
+    Output_file_name = []                            
+    if 'v' in version:
+        Input_file_name = [
+                           head_name_cat + "_"+version+'.shp',
+                           head_name_riv + "_"+version+'.shp',
+                           head_name_slake + "_"+version+'.shp',
+                           head_name_nlake + "_"+version+'.shp',
+                           ]
+        Output_file_name = [
+                           head_name_cat + "_"+version+'.geojson',
+                           head_name_riv + "_"+version+'.geojson',
+                           head_name_slake + "_"+version+'.geojson',
+                           head_name_nlake + "_"+version+'.geojson',
+                           ]
+    else:
+        Input_file_name = [
+                           head_name_cat +'.shp',
+                           head_name_riv +'.shp',
+                           head_name_slake +'.shp',
+                           head_name_nlake +'.shp',
+                           ]
+        Output_file_name = [
+                           head_name_cat +'.geojson',
+                           head_name_riv +'.geojson',
+                           head_name_slake +'.geojson',
+                           head_name_nlake +'.geojson',
+                           ]
+    created_jason_files = []   
+
+    for i  in  range(0,len(Input_file_name)):
+        input_path = os.path.join(product_dir,Input_file_name[i])
+        output_jason_path = os.path.join(product_dir,Output_file_name[i])
+        if not os.path.exists(input_path):
+            continue 
+        created_jason_files.append(output_jason_path)                   
+        # reproject to WGS84
+        input_wgs_84 = processing.run("native:reprojectlayer", {'INPUT':input_path,
+                                      'TARGET_CRS':QgsCoordinateReferenceSystem('EPSG:4326'),
+                                       'OUTPUT':"memory:"})['OUTPUT']  
+
+        # input_wgs_84_smth =  processing.run("native:smoothgeometry", {'INPUT':input_wgs_84,
+        #                                                         'ITERATIONS':5,'OFFSET':0.5,'MAX_ANGLE':180,
+        #                                                         'OUTPUT':"memory:"})['OUTPUT']
+
+        
+        if 'finalcat_info' in Input_file_name[i] or "finalcat_info_riv" in Input_file_name[i]:
+            input_tojson = processing.run("native:fieldcalculator", {'INPUT':input_wgs_84,
+                                                      'FIELD_NAME':'rvhName','FIELD_TYPE':2,'FIELD_LENGTH':20,'FIELD_PRECISION':0,
+                                                      'FORMULA':'\'sub\' +to_string(to_int(\"SubId\")) ','OUTPUT':"memory:"})['OUTPUT']
+        else:
+             input_tojson = input_tojson
+        
+        for TOLERANCE in TOLERANCEs:                               
+            input_wgs_84_simplify = processing.run("native:simplifygeometries", {
+                                                   'INPUT':input_tojson,
+                                                   'METHOD':0,
+                                                   'TOLERANCE':TOLERANCE,
+                                                   'OUTPUT':"memory:"
+                                                   }
+                                                   )['OUTPUT']
+                                               
+            qgis.core.QgsVectorFileWriter.writeAsVectorFormat(input_wgs_84_simplify,output_jason_path, 'utf-8', input_wgs_84_simplify.crs(), 'GeoJson',layerOptions=['COORDINATE_PRECISION=3'])
+            
+            json_file_size = os.stat(output_jason_path).st_size/1024/1024 #to MB
+            if json_file_size <= 20:
+                break
+    if len(created_jason_files) > 1:
+        for i in range(0,len(created_jason_files)):
+            injson = load(open(created_jason_files[i]))
+            if i == 0:
+                output_jason = injson
+            else:
+                output_jason['features'] += injson['features']
+                
+        with open(os.path.join(product_dir,'routing_product.geojson'), 'w', encoding='utf-8') as f:
+            json.dump(output_jason, f, ensure_ascii=False, indent=4)
+     
+    return 
+    
+##############
+
+
+def copy_data_and_dissolve(all_subids,tempfolder,processing,Path_Temp_final_rviply,Path_Temp_final_rvi,
+    mapoldnew_info,COLUMN_NAMES_CONSTANT_CLEAN,OutputFolder,Path_Catchment_Polygon,context,Path_final_rviply_input = '#',Path_final_rvi_input = '#'):    
+    
+    if len(all_subids) > 5000:
+
+        Path_final_rviply = os.path.join(
+            tempfolder,
+            "temp_finalriv_ply"  + ".shp",
+        )
+        Path_final_rvi = os.path.join(
+            tempfolder,
+            "temp_finalriv"  + ".shp",
+        )
+    
+        Copy_Pddataframe_to_shpfile_main(
+            Path_shpfile = Path_Temp_final_rviply,
+            Pddataframe = mapoldnew_info,
+            all_subids = all_subids,
+            tempfolder = tempfolder,
+            processing = processing,
+            output = Path_final_rviply,
+            link_col_nm_shp="SubId",
+            link_col_nm_df="Old_SubId",
+            UpdateColNM=["#"],
+        )  
+        
+        Copy_Pddataframe_to_shpfile_main(
+            Path_shpfile = Path_Temp_final_rvi,
+            Pddataframe = mapoldnew_info,
+            all_subids = all_subids,
+            tempfolder = tempfolder,
+            processing = processing,  
+            output =   Path_final_rvi,    
+            link_col_nm_shp="SubId",
+            link_col_nm_df="Old_SubId",
+            UpdateColNM=["#"],
+        )    
+                              
+    else:
+            
+        Path_final_rviply = Path_Temp_final_rviply
+        Path_final_rvi = Path_Temp_final_rvi
+                    
+        Copy_Pddataframe_to_shpfile(
+            Path_Temp_final_rviply,
+            mapoldnew_info,
+            link_col_nm_shp="SubId",
+            link_col_nm_df="Old_SubId",
+            UpdateColNM=["#"],
+        )
+        Copy_Pddataframe_to_shpfile(
+            Path_Temp_final_rvi,
+            mapoldnew_info,
+            link_col_nm_shp="SubId",
+            link_col_nm_df="Old_SubId",
+            UpdateColNM=["#"],
+        )
+
+    # dissolve shpfile based on new subid
+    if len(os.path.basename(Path_Catchment_Polygon).split('_')) == 5:
+        Path_final_rviply_out = os.path.join(OutputFolder, "finalcat_info_"+os.path.basename(Path_Catchment_Polygon).split('_')[4])
+        Path_final_rvi_out = os.path.join(OutputFolder, "finalcat_info_riv_"+os.path.basename(Path_Catchment_Polygon).split('_')[4])
+    else:
+        Path_final_rviply_out = os.path.join(OutputFolder, "finalcat_info.shp")
+        Path_final_rvi_out = os.path.join(OutputFolder, "finalcat_info_riv.shp")
+
+    if Path_final_rviply_input != '#' and Path_final_rvi_input != '#':
+        Path_final_rviply_out = os.path.join(OutputFolder, os.path.basename(Path_final_rviply_input))
+        Path_final_rvi_out = os.path.join(OutputFolder, os.path.basename(Path_final_rvi_input))
+        
+                
+    Clean_Attribute_Name(Path_final_rvi, COLUMN_NAMES_CONSTANT_CLEAN)
+    Clean_Attribute_Name(Path_final_rviply, COLUMN_NAMES_CONSTANT_CLEAN)
+    
+                    
+    qgis_vector_dissolve(
+        processing,
+        context,
+        INPUT=Path_final_rvi,
+        FIELD=["SubId"],
+        OUTPUT=Path_final_rvi_out,
+    )
+    ply_draft = qgis_vector_dissolve(
+        processing,
+        context,
+        INPUT=Path_final_rviply,
+        FIELD=["SubId"],
+        OUTPUT="memory:",
+    )["OUTPUT"]
+    
+    # clean attribute table of shpfile
+    Clean_Attribute_Name(Path_final_rvi_out, COLUMN_NAMES_CONSTANT_CLEAN)
+    ply_draft,tempnum = Clean_Attribute_Name(ply_draft, COLUMN_NAMES_CONSTANT_CLEAN,Input_Is_Feature_In_Mem = True)
+
+    # add centroid to new drived polygons
+    
+    formular = "x(centroid(transform($geometry,'%s','%s')))" % (
+        ply_draft.crs().authid(),
+        "EPSG:4326",
+    )
+
+    ply_draft = qgis_vector_field_calculator(
+        processing=processing,
+        context=context,
+        FORMULA=formular,
+        FIELD_NAME="centroid_x",
+        INPUT=ply_draft,
+        OUTPUT="memory:",
+    )["OUTPUT"]
+
+    formular = "y(centroid(transform($geometry,'%s','%s')))" % (
+        ply_draft.crs().authid(),
+        "EPSG:4326",
+    )
+
+    qgis_vector_field_calculator(
+        processing=processing,
+        context=context,
+        FORMULA=formular,
+        FIELD_NAME="centroid_y",
+        INPUT=ply_draft,
+        OUTPUT=Path_final_rviply_out,
+    )
+    if 'finalcat_info' in Path_final_rviply_out:
+        create_geo_jason_file(processing,Path_final_rviply_out)
+    return 
+
+
+def Copy_Pddataframe_to_shpfile_main(
+    Path_shpfile,
+    Pddataframe,
+    all_subids,
+    tempfolder,
+    processing,
+    output,
+    link_col_nm_shp="SubId",
+    link_col_nm_df="SubId",
+    UpdateColNM=["#"],
+    Input_Is_Feature_In_Mem=False,
+):
+
+    path_to_temp_files = []
+    ncores = 3
+    feature_per_file = int(len(all_subids)/ncores)
+    
+    for i in range(0,ncores):
+        num_random = str(np.random.randint(1, 10000 + 1)) +'_'+str(i)
+        path_of_i_temp_file = os.path.join(
+            tempfolder,
+            "temp_finalriv_ply" + num_random + ".shp",
+        )
+        if i != ncores - 1:
+            subids_of_i_temp_file = all_subids[i*feature_per_file:(i+1)*feature_per_file]
+        else:
+            subids_of_i_temp_file = all_subids[i*feature_per_file:len(all_subids)]
+                
+        Selectfeatureattributes(
+            processing,
+            Input=Path_shpfile,
+            Output=path_of_i_temp_file,
+            Attri_NM="SubId",
+            Values=subids_of_i_temp_file,
+        )
+        path_to_temp_files.append(path_of_i_temp_file)
+
+    print(path_to_temp_files)
+    Parallel(n_jobs=ncores, verbose=1, backend="threading")(
+        delayed(Copy_Pddataframe_to_shpfile)(temp_file,Pddataframe,
+        link_col_nm_shp="SubId",link_col_nm_df="Old_SubId",UpdateColNM=["#"]
+        ) for temp_file in path_to_temp_files)
+    
+    processing.run("native:mergevectorlayers", {'LAYERS':path_to_temp_files,'CRS':None,'OUTPUT':output})
+    
+    return 
 
 
 def qgis_raster_gdal_warpreproject(processing, Input, TARGET_CRS, Output):
