@@ -250,10 +250,14 @@ def simplify_routing_structure_by_drainage_area_qgis(
                 Attributes=Conn_Lakes_ply,
             )
         else:
+            if len(os.path.basename(Path_Conl_ply).split('_')) == 4:
+                outlake_name = 'sl_non_connected_lake_' + os.path.basename(Path_Conl_ply).split('_')[3]
+            else:
+                outlake_name = 'sl_non_connected_lake.shp'
             Selectfeatureattributes(
                 processing,
                 Input=Path_Conl_ply,
-                Output=os.path.join(outputfolder_subid, 'sl_non_connected_lake_' + os.path.basename(Path_Conl_ply).split('_')[3]),
+                Output=os.path.join(outputfolder_subid, outlake_name),
                 Attri_NM="Hylak_id",
                 Values=Conn_To_NonConlakeids,
             )
