@@ -9,6 +9,7 @@ from arcgis.features import GeoAccessor, GeoSeriesAccessor
 import arcpy
 from arcpy import env
 from arcpy.sa import *
+import shutil
 
 
 import sys, os
@@ -20,10 +21,7 @@ arcpy.env.overwriteOutput = True
 arcpy.CheckOutExtension("Spatial")
 
 def simplify_routing_structure_by_drainage_area_arcgis(
-    Path_final_riv_ply="#",
-    Path_final_riv="#",
-    Path_Con_Lake_ply="#",
-    Path_NonCon_Lake_ply="#",
+    Routing_Product_Folder= '#',
     Area_Min=-1,
     OutputFolder="#",
 ):
@@ -100,6 +98,44 @@ def simplify_routing_structure_by_drainage_area_arcgis(
     if not os.path.exists(tempfolder):
         os.makedirs(tempfolder)
 
+    Path_Catchment_Polygon="#"
+    Path_River_Polyline="#"
+    Path_Con_Lake_ply="#"
+    Path_NonCon_Lake_ply="#"
+    Path_obs_gauge_point="#"
+    Path_final_cat_ply="#"
+    Path_final_cat_riv="#"
+
+    ##define input files from routing prodcut 
+    for file in os.listdir(Routing_Product_Folder):
+        if file.endswith(".shp"):
+            if 'catchment_without_merging_lakes' in file:
+                Path_Catchment_Polygon = os.path.join(Routing_Product_Folder, file)
+            if 'river_without_merging_lakes' in file:
+                Path_River_Polyline = os.path.join(Routing_Product_Folder, file)
+            if 'sl_connected_lake' in file:
+                Path_Con_Lake_ply = os.path.join(Routing_Product_Folder, file)
+            if 'sl_non_connected_lake' in file:
+                Path_NonCon_Lake_ply = os.path.join(Routing_Product_Folder, file)
+            if 'obs_gauges' in file:
+                Path_obs_gauge_point = os.path.join(Routing_Product_Folder, file)
+            if 'finalcat_info' in file:
+                Path_final_cat_ply = os.path.join(Routing_Product_Folder, file)
+            if 'finalcat_info_riv' in file:
+                Path_final_cat_riv = os.path.join(Routing_Product_Folder, file)                
+
+    if Path_Catchment_Polygon == '#' or  Path_River_Polyline =='#':
+        print("Invalid routing product folder ")
+
+    Path_final_riv_ply = Path_Catchment_Polygon
+    Path_final_riv = Path_River_Polyline
+
+    ## copy obs_gauges to output folder 
+    for file in os.listdir(Routing_Product_Folder):
+        if 'obs_gauges' in file:
+            shutil.copy(os.path.join(Routing_Product_Folder, file), os.path.join(OutputFolder, file))
+
+
     # overall procedure,
     # 1. first get product attribute table
     # 2. determine which features needs to be merged together to increage
@@ -146,13 +182,15 @@ def simplify_routing_structure_by_drainage_area_arcgis(
         Path_final_riv = os.path.join(tempfolder,'selected_riv.shp'),
     )
    
-    # export lakes     
-    Conn_Lakes_ply = pd.DataFrame.spatial.from_featureclass(Path_Conl_ply)
-    non_conn_Lakes_ply = pd.DataFrame.spatial.from_featureclass(Path_Non_ConL_ply)
-    
-    lake_mask = Conn_Lakes_ply['Hylak_id'].isin(Connected_Lake_Mainriv)
-    Conn_Lakes_ply_select = Conn_Lakes_ply.loc[lake_mask].copy()
-    Conn_Lakes_ply_not_select = Conn_Lakes_ply.loc[np.logical_not(lake_mask)].copy()
+    # export lakes 
+    if Path_Conl_ply == '#':
+        Conn_Lakes_ply_select = []
+        Conn_Lakes_ply_not_select = []
+    else:     
+        Conn_Lakes_ply = pd.DataFrame.spatial.from_featureclass(Path_Conl_ply)    
+        lake_mask = Conn_Lakes_ply['Hylak_id'].isin(Connected_Lake_Mainriv)
+        Conn_Lakes_ply_select = Conn_Lakes_ply.loc[lake_mask].copy()
+        Conn_Lakes_ply_not_select = Conn_Lakes_ply.loc[np.logical_not(lake_mask)].copy()
 
     # export lake polygons
     # export connected lake polygon
@@ -165,8 +203,15 @@ def simplify_routing_structure_by_drainage_area_arcgis(
         new_non_connected_lake = pd.concat([non_conn_Lakes_ply, Conn_Lakes_ply_not_select], ignore_index=True)
         new_non_connected_lake.spatial.to_featureclass(location=os.path.join(OutputFolder,os.path.basename(Path_NonCon_Lake_ply)),overwrite=True,sanitize_columns=False)
     if len(Conn_Lakes_ply_not_select) <= 0 and Path_NonCon_Lake_ply != '#':
+        non_conn_Lakes_ply = pd.DataFrame.spatial.from_featureclass(Path_NonCon_Lake_ply)
         non_conn_Lakes_ply.spatial.to_featureclass(location=os.path.join(OutputFolder,os.path.basename(Path_NonCon_Lake_ply)),overwrite=True,sanitize_columns=False)
     if len(Conn_Lakes_ply_not_select) > 0 and Path_NonCon_Lake_ply == '#':
-        Conn_Lakes_ply_not_select.spatial.to_featureclass(location=os.path.join(OutputFolder,os.path.basename(Path_NonCon_Lake_ply)),overwrite=True,sanitize_columns=False)
+        
+        if len(os.path.basename(Path_Conl_ply).split('_')) == 4:
+            outlake_name = 'sl_non_connected_lake_' + os.path.basename(Path_Conl_ply).split('_')[3]
+        else:
+            outlake_name = 'sl_non_connected_lake.shp'
+            
+        Conn_Lakes_ply_not_select.spatial.to_featureclass(location=os.path.join(OutputFolder,outlake_name),overwrite=True,sanitize_columns=False)
         
     return 
